@@ -68,7 +68,6 @@ struct Traits<Array2DDiagonal<Type_> >
     typedef Array2DDiagonal<Type_> SubVector;
 
     typedef Type_                Type;
-    typedef typename RemoveConst<Type_>::Type const& ReturnType;
     typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
     enum
@@ -93,7 +92,7 @@ struct Traits<Array2DDiagonal<Type_> >
   * The range of the rows and the columns is the same.
   **/
 template<class Type_>
-class Array2DDiagonal : public IArray2D< Array2DDiagonal<Type_> >
+class Array2DDiagonal: public IArray2D< Array2DDiagonal<Type_> >
 {
   public:
     /** Type for the Interface Class. */
@@ -108,7 +107,7 @@ class Array2DDiagonal : public IArray2D< Array2DDiagonal<Type_> >
     typedef typename hidden::Traits<Array2DDiagonal<Type_> >::SubArray SubArray;
 
     typedef typename hidden::Traits<Array2DDiagonal<Type_> >::Type Type;
-    typedef typename hidden::Traits<Array2DDiagonal<Type_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits<Array2DDiagonal<Type_> >::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -134,12 +133,12 @@ class Array2DDiagonal : public IArray2D< Array2DDiagonal<Type_> >
      *  @param T the container to copy
      *  @param ref true if T is wrapped
      **/
-    Array2DDiagonal( Array2DDiagonal const& T, bool ref=false) : Base(T, ref) {}
+    Array2DDiagonal( Array2DDiagonal const& T, bool ref=false): Base(T, ref) {}
     /** constructor by reference, ref_=1 in the range given by I.
      *  @param T the Container to wrap
      *  @param I range of the container to wrap
      **/
-    Array2DDiagonal( Array2DDiagonal const& T, Range const& I) : Base(T, I, I) {}
+    Array2DDiagonal( Array2DDiagonal const& T, Range const& I): Base(T, I, I) {}
     /** Copy constructor using an expression.
      *  @param T the container to wrap
      **/
