@@ -102,8 +102,7 @@ struct Traits< Const::Identity <Type_, Size_> >
   };
 
   typedef Type_                Type;
-  typedef typename RemoveConst<Type_>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;
 };
 /** @ingroup hidden
  *  @brief Traits class for the identity constant matrix
@@ -120,8 +119,7 @@ struct Traits< Const::Vector <Type_, Size_> >
     storage_   = Arrays::sparse_
   };
   typedef Type_                Type;
-  typedef typename RemoveConst<Type_>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;
 };
 /** @ingroup hidden
  *  @brief Traits class for the constant vector_
@@ -138,8 +136,7 @@ struct Traits< Const::Point <Type_, Size_> >
     storage_   = Arrays::sparse_
   };
   typedef Type_                Type;
-  typedef typename RemoveConst<Type_>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;
 };
 /** @ingroup hidden
  *  @brief Traits class for the square constant matrix
@@ -156,8 +153,7 @@ struct Traits< Const::Square <Type_, Size_> >
     storage_   = Arrays::sparse_
   };
   typedef Type_                Type;
-  typedef typename RemoveConst<Type_>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;
 };
 /** @ingroup hidden
  *  @brief Traits class for the general constant matrix
@@ -174,8 +170,7 @@ struct Traits< Const::Array<Type_, SizeRows_, SizeCols_> >
     storage_   = Arrays::sparse_
   };
   typedef Type_                Type;
-  typedef typename RemoveConst<Type_>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;
 };
 /** @ingroup hidden
  *  @brief Traits class for the upper triangular constant matrix
@@ -192,8 +187,7 @@ struct Traits< Const::UpperTriangular<Type_, SizeRows_, SizeCols_> >
     storage_   = Arrays::sparse_
   };
   typedef Type_                Type;
-  typedef typename RemoveConst<Type_>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;
 };
 /** @ingroup hidden
  *  @brief Traits class for the lower triangular constant matrix
@@ -210,8 +204,7 @@ struct Traits< Const::LowerTriangular<Type_, SizeRows_, SizeCols_> >
     storage_   = Arrays::sparse_
   };
   typedef Type_                Type;
-  typedef typename RemoveConst<Type_>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;
 };
 
 } // namespace hidden
@@ -237,12 +230,13 @@ class IConstArray: protected IContainer2D<hidden::Traits<Derived>::sizeRows_, hi
     /** Type for the IContainer2D base Class. */
     typedef IContainer2D<sizeRows_, sizeCols_ > Base2D;
     typedef ExprBase<Derived> Base;
+    typedef typename hidden::Traits< Derived >::ConstReturnType ConstReturnType;
 
   protected:
     /** default constructor */
     IConstArray(): Base2D(), Base() {}
     /** constructor with specified dimension */
-    IConstArray(Range const& rows, Range const& cols) : Base2D(rows, cols), Base() {}
+    IConstArray(Range const& rows, Range const& cols): Base2D(rows, cols), Base() {}
     /** destructor */
     ~IConstArray() {}
 
@@ -301,12 +295,12 @@ class IConstArray: protected IContainer2D<hidden::Traits<Derived>::sizeRows_, hi
  * @tparam Size_ the size of the identity ArrayXX. Default is UnknownSize.
  **/
 template< typename Type_, int Size_ >
-class Identity : public IConstArray<Identity<Type_, Size_> >
+class Identity: public IConstArray<Identity<Type_, Size_> >
 {
   public:
     typedef IConstArray<Identity<Type_, Size_> > Base;
     typedef typename hidden::Traits< Identity<Type_, Size_> >::Type Type;
-    typedef typename hidden::Traits< Identity<Type_, Size_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits< Identity<Type_, Size_> >::ConstReturnType ConstReturnType;
     enum
     {
       structure_ = hidden::Traits< Identity <Type_, Size_> >::structure_,
@@ -316,19 +310,17 @@ class Identity : public IConstArray<Identity<Type_, Size_> >
       storage_   = hidden::Traits< Identity <Type_, Size_> >::storage_
     };
     /** default constructor */
-    Identity() : Base() {}
+    Identity(): Base() {}
     /** constructor with specified dimension */
-    Identity(Range I) : Base(I,I) {}
+    Identity(Range I): Base(I,I) {}
     /** @return the element (i,j) of the identity matrix.
-     *  @param i index of the row
-     *  @param j index of the column
+     *  @param i,j row and column indexes
      **/
-    inline ReturnType elt2Impl(int i, int j) const
-    { return (i==j ? Type(1) : Type(0));}
+    inline ConstReturnType elt2Impl(int i, int j) const { return (i==j ? Type(1) : Type(0));}
     /** @return the element ith element of the identity matrix
      *  @param i index of the ith element
      **/
-    inline ReturnType elt1Impl(int i) const { return Type(1);}
+    inline ConstReturnType elt1Impl(int i) const { return Type(1);}
 };
 
 /**@ingroup Arrays
@@ -354,12 +346,12 @@ class Identity : public IConstArray<Identity<Type_, Size_> >
  * @tparam Size_ the size of the square ArrayXX. Default is UnknownSize.
  **/
 template< typename Type_, int Size_ >
-class Square : public IConstArray<Square<Type_, Size_> >
+class Square: public IConstArray<Square<Type_, Size_> >
 {
   public:
     typedef IConstArray<Square<Type_, Size_> > Base;
     typedef typename hidden::Traits< Square<Type_, Size_> >::Type Type;
-    typedef typename hidden::Traits< Square<Type_, Size_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits< Square<Type_, Size_> >::ConstReturnType ConstReturnType;
     enum
     {
       structure_ = hidden::Traits< Square <Type_, Size_> >::structure_,
@@ -369,14 +361,13 @@ class Square : public IConstArray<Square<Type_, Size_> >
       storage_   = hidden::Traits< Square <Type_, Size_> >::storage_
     };
     /** default constructor */
-    Square() : Base() {}
+    Square(): Base() {}
     /** constructor with specified dimension */
-    Square(Range I) : Base(I, I) {}
+    Square(Range I): Base(I, I) {}
     /** @return the element (i,j) of the constant square matrix.
-     *  @param i index of the row
-     *  @param j index of the column
+     *  @param i,j row and column indexes
      **/
-    inline ReturnType elt2Impl(int i, int j) const { return (Type(1));}
+    inline ConstReturnType elt2Impl(int i, int j) const { return (Type(1));}
 };
 
 /**@ingroup Arrays
@@ -403,12 +394,12 @@ class Square : public IConstArray<Square<Type_, Size_> >
  * @tparam SizeCols_ the number of column of the matrix. Default is UnknownSize.
  **/
 template< typename Type_, int SizeRows_, int SizeCols_ >
-class Array : public IConstArray<Array<Type_, SizeRows_, SizeCols_> >
+class Array: public IConstArray<Array<Type_, SizeRows_, SizeCols_> >
 {
   public:
     typedef IConstArray<Array<Type_, SizeRows_, SizeCols_> > Base;
     typedef typename hidden::Traits< Array<Type_, SizeRows_, SizeCols_> >::Type Type;
-    typedef typename hidden::Traits< Array<Type_, SizeRows_, SizeCols_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits< Array<Type_, SizeRows_, SizeCols_> >::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -419,14 +410,13 @@ class Array : public IConstArray<Array<Type_, SizeRows_, SizeCols_> >
       storage_   = hidden::Traits< Array <Type_, SizeRows_, SizeCols_> >::storage_
     };
     /** default constructor */
-    Array() : Base() {}
+    Array(): Base() {}
     /** constructor with specified dimension */
-    Array(Range rangeRows, Range rangeCols) : Base(rangeRows, rangeCols) {}
+    Array(Range rangeRows, Range rangeCols): Base(rangeRows, rangeCols) {}
     /** @return the element (i,j) of the constant square matrix.
-     *  @param i index of the row
-     *  @param j index of the column
+     *  @param i,j row and column indexes
      **/
-    inline ReturnType elt2Impl(int i, int j) const { return (Type(1));}
+    inline ConstReturnType elt2Impl(int i, int j) const { return (Type(1));}
 };
 
 /**@ingroup Arrays
@@ -453,12 +443,12 @@ class Array : public IConstArray<Array<Type_, SizeRows_, SizeCols_> >
  * @tparam SizeCols_ the number of column of the matrix. Default is UnknownSize.
  **/
 template< typename Type_, int SizeRows_, int SizeCols_ >
-class UpperTriangular : public IConstArray<UpperTriangular<Type_, SizeRows_, SizeCols_> >
+class UpperTriangular: public IConstArray<UpperTriangular<Type_, SizeRows_, SizeCols_> >
 {
   public:
     typedef IConstArray<UpperTriangular<Type_, SizeRows_, SizeCols_> > Base;
     typedef typename hidden::Traits< UpperTriangular<Type_, SizeRows_, SizeCols_> >::Type Type;
-    typedef typename hidden::Traits< UpperTriangular<Type_, SizeRows_, SizeCols_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits< UpperTriangular<Type_, SizeRows_, SizeCols_> >::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -469,14 +459,13 @@ class UpperTriangular : public IConstArray<UpperTriangular<Type_, SizeRows_, Siz
       storage_   = hidden::Traits< UpperTriangular <Type_, SizeRows_, SizeCols_> >::storage_
     };
     /** default constructor */
-    UpperTriangular() : Base() {}
+    UpperTriangular(): Base() {}
     /** constructor with specified dimension */
-    UpperTriangular(Range rangeRows, Range rangeCols) : Base(rangeRows, rangeCols) {}
+    UpperTriangular(Range rangeRows, Range rangeCols): Base(rangeRows, rangeCols) {}
     /** @return the element (i,j) of the constant upper triangular matrix.
-     *  @param i index of the row
-     *  @param j index of the column
+     *  @param i,j row and column indexes
      **/
-    inline ReturnType elt2Impl(int i, int j) const { return (Type(1));}
+    inline ConstReturnType elt2Impl(int i, int j) const { return i<j ? (Type(1)) : Type(0);}
 };
 
 /**@ingroup Arrays
@@ -503,12 +492,12 @@ class UpperTriangular : public IConstArray<UpperTriangular<Type_, SizeRows_, Siz
  * @tparam SizeCols_ the number of column of the matrix. Default is UnknownSize.
  **/
 template< typename Type_, int SizeRows_, int SizeCols_ >
-class LowerTriangular : public IConstArray<LowerTriangular<Type_, SizeRows_, SizeCols_> >
+class LowerTriangular: public IConstArray<LowerTriangular<Type_, SizeRows_, SizeCols_> >
 {
   public:
     typedef IConstArray<LowerTriangular<Type_, SizeRows_, SizeCols_> > Base;
     typedef typename hidden::Traits< LowerTriangular<Type_, SizeRows_, SizeCols_> >::Type Type;
-    typedef typename hidden::Traits< LowerTriangular<Type_, SizeRows_, SizeCols_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits< LowerTriangular<Type_, SizeRows_, SizeCols_> >::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -519,14 +508,13 @@ class LowerTriangular : public IConstArray<LowerTriangular<Type_, SizeRows_, Siz
       storage_   = hidden::Traits< LowerTriangular <Type_, SizeRows_, SizeCols_> >::storage_
     };
     /** default constructor */
-    LowerTriangular() : Base() {}
+    LowerTriangular(): Base() {}
     /** constructor with specified dimension */
-    LowerTriangular(Range rangeRows, Range rangeCols) : Base(rangeRows, rangeCols) {}
+    LowerTriangular(Range rangeRows, Range rangeCols): Base(rangeRows, rangeCols) {}
     /** @return the element (i,j) of the constant lower triangular matrix.
-     *  @param i index of the row
-     *  @param j index of the column
+     *  @param i,j row and column indexes
      **/
-    inline ReturnType elt2Impl(int i, int j) const { return (Type(1));}
+    inline ConstReturnType elt2Impl(int i, int j) const { return (Type(1));}
 };
 
 /**@ingroup Arrays
@@ -548,12 +536,12 @@ class LowerTriangular : public IConstArray<LowerTriangular<Type_, SizeRows_, Siz
  * @tparam Size_ the size of the row-vector. Default is UnknownSize.
  **/
 template< typename Type_, int Size_ >
-class Point : public IConstArray<Point<Type_, Size_> >
+class Point: public IConstArray<Point<Type_, Size_> >
 {
   public:
     typedef IConstArray<Point<Type_, Size_> > Base;
     typedef typename hidden::Traits< Point<Type_, Size_> >::Type Type;
-    typedef typename hidden::Traits< Point<Type_, Size_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits< Point<Type_, Size_> >::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -564,18 +552,17 @@ class Point : public IConstArray<Point<Type_, Size_> >
       storage_   = hidden::Traits< Point <Type_, Size_> >::storage_
     };
     /** default constructor */
-    Point() : Base() {}
+    Point(): Base() {}
     /** constructor with specified dimension */
-    Point(Range I) : Base(1, I) {}
+    Point(Range I): Base(1, I) {}
     /** @return the j-th element  of the constant row-vector.
      *  @param j index of the element
      **/
-    inline ReturnType elt1Impl(int j) const { return (Type(1));}
+    inline ConstReturnType elt1Impl(int j) const { return (Type(1));}
     /** @return the element (i,j) of the constant row-vector.
-     *  @param i index of the row
-     *  @param j index of the column
+     *  @param i,j row and column indexes
      **/
-    inline ReturnType elt2Impl(int i, int j) const { return (Type(1));}
+    inline ConstReturnType elt2Impl(int i, int j) const { return (Type(1));}
 };
 
 /**@ingroup Arrays
@@ -597,12 +584,12 @@ class Point : public IConstArray<Point<Type_, Size_> >
  * @tparam Size_ the size of the row-vector. Default is UnknownSize.
  **/
 template< typename Type_, int Size_ >
-class Vector : public IConstArray<Vector<Type_, Size_> >
+class Vector: public IConstArray<Vector<Type_, Size_> >
 {
   public:
     typedef IConstArray<Vector<Type_, Size_> > Base;
     typedef typename hidden::Traits< Vector<Type_, Size_> >::Type Type;
-    typedef typename hidden::Traits< Vector<Type_, Size_> >::ReturnType ReturnType;
+    typedef typename hidden::Traits< Vector<Type_, Size_> >::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -613,17 +600,17 @@ class Vector : public IConstArray<Vector<Type_, Size_> >
       storage_   = hidden::Traits< Vector <Type_, Size_> >::storage_
     };
     /** default constructor */
-    Vector() : Base() {}
+    Vector(): Base() {}
     /** constructor with specified dimension */
-    Vector(Range I) : Base(I, 1) {}
+    Vector(Range I): Base(I, 1) {}
     /** @return the i-th element of the constant vector.
      *  @param i index of the element
      **/
-    inline ReturnType elt1Impl(int i) const { return (Type(1));}
+    inline ConstReturnType elt1Impl(int i) const { return (Type(1));}
     /** @return the element (i,j) of the constant vector.
      *  @param i,j row and column indexes
      **/
-    inline ReturnType elt2Impl(int i, int j) const { return (Type(1));}
+    inline ConstReturnType elt2Impl(int i, int j) const { return (Type(1));}
 };
 
 
