@@ -66,14 +66,13 @@ struct ProductTraits<Lhs, Rhs, Arrays::array2D_, RStructure_>
     // if there is more block on the left side, we use bp --> result is by_col_
     orient_   =  ( Traits<Lhs>::sizeRows_ == UnknownSize || Traits<Rhs>::sizeCols_ == UnknownSize)
                  ? Traits<Rhs>::orient_
-                 : (Traits<Lhs>::sizeRows_)/blockSize < (Traits<Rhs>::sizeCols_)/blockSize
+                 : (Traits<Lhs>::sizeRows_)/blockSize_ < (Traits<Rhs>::sizeCols_)/blockSize_
                    ? int(Arrays::by_row_) : int(Arrays::by_col_),
     storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
                 ? int(Arrays::dense_) : int(Arrays::sparse_)
   };
 
   typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
   typedef typename RemoveConst<Type>::Type const& ConstReturnType;
   typedef CAllocator<Type, sizeRows_, sizeCols_, orient_> Allocator;
 };
@@ -91,13 +90,12 @@ struct ProductTraits<Lhs, Rhs, Arrays::square_, RStructure_>
     // if there is more block on the left side, we use bp --> result is by_col_
     orient_   =  ( Traits<Lhs>::sizeRows_ == UnknownSize || Traits<Rhs>::sizeCols_ == UnknownSize)
                  ? Traits<Rhs>::orient_
-                 : (Traits<Lhs>::sizeRows_)/blockSize < (Traits<Rhs>::sizeCols_)/blockSize
+                 : (Traits<Lhs>::sizeRows_)/blockSize_ < (Traits<Rhs>::sizeCols_)/blockSize_
                    ? int(Arrays::by_row_) : int(Arrays::by_col_),
     storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
               ? int(Arrays::dense_) : int(Arrays::sparse_)
   };
   typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
   typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
   typedef CAllocator<Type, sizeRows_,sizeCols_, orient_> Allocator;
@@ -114,13 +112,12 @@ struct ProductTraits<Lhs, Rhs, Arrays::square_, Arrays::square_>
     // if there is more block on the left side, we use bp --> result is by_col_
     orient_   =  ( Traits<Lhs>::sizeRows_ == UnknownSize || Traits<Rhs>::sizeCols_ == UnknownSize)
                  ? Traits<Rhs>::orient_
-                 : (Traits<Lhs>::sizeRows_)/blockSize < (Traits<Rhs>::sizeCols_)/blockSize
+                 : (Traits<Lhs>::sizeRows_)/blockSize_ < (Traits<Rhs>::sizeCols_)/blockSize_
                    ? int(Arrays::by_row_) : int(Arrays::by_col_),
     storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
               ? int(Arrays::dense_) : int(Arrays::sparse_)
   };
   typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
   typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
   typedef CAllocator<Type, sizeRows_, sizeCols_, orient_> Allocator;
@@ -139,13 +136,12 @@ struct ProductTraits<Lhs, Rhs, Arrays::lower_triangular_, RStructure_>
     // if there is more block on the left side, we use bp --> result is by_col_
     orient_   =  ( sizeRows_ == UnknownSize || sizeCols_ == UnknownSize)
                  ? Traits<Rhs>::orient_
-                 : sizeRows_/blockSize < sizeCols_/blockSize
+                 : sizeRows_/blockSize_ < sizeCols_/blockSize_
                    ? int(Arrays::by_row_) : int(Arrays::by_col_),
     storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
               ? int(Arrays::dense_) : int(Arrays::sparse_)
   };
   typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
   typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
   typedef CAllocator<Type, sizeRows_, sizeCols_, orient_> Allocator;
@@ -162,13 +158,13 @@ struct ProductTraits<Lhs, Rhs, Arrays::lower_triangular_, Arrays::lower_triangul
     // if there is more block on the left side, we use bp --> result is by_col_
     orient_   =  ( Traits<Lhs>::sizeRows_ == UnknownSize || Traits<Rhs>::sizeCols_ == UnknownSize)
                  ? Traits<Rhs>::orient_
-                 : (Traits<Lhs>::sizeRows_)/blockSize < (Traits<Rhs>::sizeCols_)/blockSize
+                 : (Traits<Lhs>::sizeRows_)/blockSize_ < (Traits<Rhs>::sizeCols_)/blockSize_
                    ? int(Arrays::by_row_) : int(Arrays::by_col_),
     storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
               ? int(Arrays::dense_) : int(Arrays::sparse_)
   };
   typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
+  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
   typedef Array2DLowerTriangular<Type> Allocator; // no CAllocator
 };
@@ -186,13 +182,12 @@ struct ProductTraits<Lhs, Rhs, Arrays::upper_triangular_, RStructure_>
     // if there is more block on the left side, we use bp --> result is by_col_
     orient_   =  ( sizeRows_ == UnknownSize || sizeCols_ == UnknownSize)
                  ? Traits<Rhs>::orient_
-                 : (Traits<Lhs>::sizeRows_)/blockSize < (Traits<Rhs>::sizeCols_)/blockSize
+                 : (Traits<Lhs>::sizeRows_)/blockSize_ < (Traits<Rhs>::sizeCols_)/blockSize_
                    ? int(Arrays::by_row_) : int(Arrays::by_col_),
     storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
               ? int(Arrays::dense_) : int(Arrays::sparse_)
   };
   typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
   typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
   typedef CAllocator<Type, sizeRows_ , sizeCols_, orient_> Allocator;
@@ -209,13 +204,12 @@ struct ProductTraits<Lhs, Rhs, Arrays::upper_triangular_, Arrays::upper_triangul
     // if there is more block on the left side, we use bp --> result is by_col_
     orient_   =  ( Traits<Lhs>::sizeRows_ == UnknownSize || Traits<Rhs>::sizeCols_ == UnknownSize)
                  ? Traits<Rhs>::orient_
-                 : (Traits<Lhs>::sizeRows_)/blockSize < (Traits<Rhs>::sizeCols_)/blockSize
+                 : (Traits<Lhs>::sizeRows_)/blockSize_ < (Traits<Rhs>::sizeCols_)/blockSize_
                    ? int(Arrays::by_row_) : int(Arrays::by_col_),
     storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
               ? int(Arrays::dense_) : int(Arrays::sparse_)
   };
   typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
   typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
   typedef Array2DUpperTriangular<Type> Allocator;  // no CAllocator
@@ -330,130 +324,34 @@ struct Traits< ArrayByDiagonalProduct < Lhs, Rhs> >
     storage_   = Lhs::storage_
   };
   typedef typename Promote<typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
-};
-/** @ingroup hidden
- *  @brief Traits class for the DiagonalByArrayProduct class
- */
-template< typename Lhs, typename Rhs>
-struct Traits< DiagonalByArrayProduct < Lhs, Rhs> >
-{
-  enum
-  {
-    structure_ = Rhs::structure_,
-    sizeRows_  = Lhs::sizeRows_,
-    sizeCols_  = Rhs::sizeCols_,
-    orient_    = Rhs::orient_,
-    storage_   = Rhs::storage_
-  };
-  typedef typename Promote<typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
-};
-/** @ingroup hidden
- *  @brief Traits class for the PointByArrayProduct
- */
-template< typename Lhs, typename Rhs>
-struct Traits< PointByArrayProduct < Lhs, Rhs> >
-{
-  enum
-   {
-     structure_ = Arrays::point_,
-     sizeRows_  = 1,
-     sizeCols_  = Traits<Rhs>::sizeCols_,
-     orient_    = Arrays::by_row_,
-     storage_   = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
-                  ? int(Arrays::dense_) : int(Arrays::sparse_)
-   };
-
-  typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
-
-  typedef CAllocator<Type, sizeRows_, sizeCols_, orient_> Allocator;
+  typedef typename RemoveConst<Type>::Type ConstReturnType; // not a reference as it is a temporary
 };
 
-/** @ingroup hidden
- *  @brief Traits class for the ArrayByVectorProduct class
- */
-template< typename Lhs, typename Rhs>
-struct Traits< ArrayByVectorProduct < Lhs, Rhs> >
-{
-  enum
-   {
-     structure_ = Arrays::vector_,
-     orient_    = Arrays::by_col_,
-     sizeRows_  = Traits<Lhs>::sizeRows_,
-     sizeCols_  = 1,
-     storage_   = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
-                  ? int(Arrays::dense_) : int(Arrays::sparse_)
-   };
-
-  typedef ProductTraits<Lhs, Rhs, Lhs::structure_, Rhs::structure_> Base;
-  typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
-
-  typedef CAllocator<Type, sizeRows_, sizeCols_, orient_> Allocator;
-};
-/** @ingroup hidden
- *  @brief Traits class for the DiagonalByArrayProduct class
- */
-template< typename Lhs, typename Rhs>
-struct Traits< VectorByPointProduct < Lhs, Rhs> >
-{
-  enum
-  {
-    structure_ = Arrays::array2D_,
-    sizeRows_  = Traits<Lhs>::sizeRows_,
-    sizeCols_  = Traits<Rhs>::sizeCols_,
-    orient_    = Arrays::by_col_,
-    storage_   = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
-                 ? int(Arrays::dense_) : int(Arrays::sparse_)
-  };
-  typedef typename Promote<typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef Type ReturnType;
-  typedef Type ConstReturnType;
-};
-/** @ingroup hidden
- *  @brief Traits class for the ArrayByArrayProduct class
- */
-template< typename Lhs, typename Rhs>
-struct Traits< ArrayByArrayProduct<Lhs, Rhs> >
-{
-  typedef ProductTraits<Lhs, Rhs, Traits<Lhs>::structure_, Traits<Rhs>::structure_> Base;
-
-  enum
-  {
-    structure_ = Base::structure_,
-    sizeRows_ = Traits<Lhs>::sizeRows_,
-    sizeCols_ = Traits<Rhs>::sizeCols_,
-    // if there is more block on the left side, we use bp --> result is by_col_
-    orient_   =  ( Traits<Lhs>::sizeRows_ == UnknownSize || Traits<Rhs>::sizeCols_ == UnknownSize)
-                 ? Traits<Rhs>::orient_
-                 : (Traits<Lhs>::sizeRows_)/blockSize < (Traits<Rhs>::sizeCols_)/blockSize
-                   ? int(Arrays::by_row_) : int(Arrays::by_col_),
-    storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
-                  ? int(Arrays::dense_) : int(Arrays::sparse_)
-  };
-  typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
-  typedef typename RemoveConst<Type>::Type const& ReturnType;
-  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
-
-  typedef typename Base::Allocator Allocator;
-};
-
-} // end namespace hidden
+} // namespace hidden
 
+/** @ingroup Arrays
+  * @brief Generic expression where a product operator is applied to two expressions
+  *
+  * @tparam Lhs left-hand side type
+  * @tparam Rhs right-hand side type
+  *
+  * This class represents an expression where a product operator is applied to
+  * two expressions.
+  *
+  * It is the return type of product operator when the left hand side expression
+  * is an array and the right and side expression is a diagonal expression.
+  *
+  * Most of the time, this is the only way that it is used, so you typically
+  * don't have to name ArrayByDiagonalProduct types explicitly.
+  **/
 template<typename Lhs, typename Rhs>
-class ArrayByDiagonalProduct : public ExprBase< ArrayByDiagonalProduct<Lhs, Rhs> >
-                             , public TRef<1>
+class ArrayByDiagonalProduct: public ExprBase< ArrayByDiagonalProduct<Lhs, Rhs> >
+                            , public TRef<1>
 {
   public:
     typedef ExprBase< ArrayByDiagonalProduct<Lhs, Rhs> > Base;
     typedef typename hidden::Traits<ArrayByDiagonalProduct>::Type Type;
-    typedef typename hidden::Traits<ArrayByDiagonalProduct>::ReturnType ReturnType;
+    typedef typename hidden::Traits<ArrayByDiagonalProduct>::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -469,7 +367,7 @@ class ArrayByDiagonalProduct : public ExprBase< ArrayByDiagonalProduct<Lhs, Rhs>
     typedef TRange<sizeCols_> ColRange;
 
     inline ArrayByDiagonalProduct( const Lhs& lhs, const Rhs& rhs)
-                                  : Base(), lhs_(lhs), rhs_(rhs)
+                                 : Base(), lhs_(lhs), rhs_(rhs)
     {
       if (lhs.cols() != rhs.rows())
       { STKRUNTIME_ERROR_NO_ARG(ArrayByDiagonalProduct, sizes mismatch);}
@@ -477,13 +375,15 @@ class ArrayByDiagonalProduct : public ExprBase< ArrayByDiagonalProduct<Lhs, Rhs>
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return lhs_.rows();}
     /** @return the columns range */
-    inline ColRange const&colsImpl() const { return rhs_.cols();}
+    inline ColRange const& colsImpl() const { return rhs_.cols();}
 
-    inline ReturnType elt2Impl(int i, int j) const { return lhs_.elt(i,j)*rhs_.elt(j);}
-    /** access to the ith element */
-    inline ReturnType elt1Impl(int i) const { return lhs_.elt(i)*rhs_.elt(i);}
-    /** access to the element */
-    inline ReturnType elt0Impl() const { return lhs_.elt()*rhs_.elt();}
+    /** @return the element (i,j) */
+    inline ConstReturnType elt2Impl(int i, int j) const { return lhs_.elt(i,j)*rhs_.elt(j);}
+    /** @return ith element
+     *  @note make sense if lhs_ is a diagonal matrix, in this case this operator is also a
+     *  diagonal matrix.
+     **/
+    inline ConstReturnType elt1Impl(int i) const { return lhs_.elt(i)*rhs_.elt(i);}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
@@ -495,14 +395,74 @@ class ArrayByDiagonalProduct : public ExprBase< ArrayByDiagonalProduct<Lhs, Rhs>
     Rhs const& rhs_;
 };
 
+namespace hidden
+{
+/** @ingroup hidden
+ *  @brief Traits class for the VectorByPointProduct class
+ */
+template< typename Lhs, typename Rhs>
+struct Traits< VectorByPointProduct < Lhs, Rhs> >
+{
+  enum
+  {
+    structure_ = Arrays::array2D_,
+    sizeRows_  = Traits<Lhs>::sizeRows_,
+    sizeCols_  = Traits<Rhs>::sizeCols_,
+    orient_    = Arrays::by_col_,
+    storage_   = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
+                 ? int(Arrays::dense_) : int(Arrays::sparse_)
+  };
+  typedef typename Promote<typename Lhs::Type, typename Rhs::Type>::result_type Type;
+  typedef Type ConstReturnType;
+};
+
+} // namespace hidden
+
+namespace hidden
+{
+/** @ingroup hidden
+ *  @brief Traits class for the DiagonalByArrayProduct class
+ */
+template< typename Lhs, typename Rhs>
+struct Traits< DiagonalByArrayProduct < Lhs, Rhs> >
+{
+  enum
+  {
+    structure_ = Rhs::structure_,
+    sizeRows_  = Lhs::sizeRows_,
+    sizeCols_  = Rhs::sizeCols_,
+    orient_    = Rhs::orient_,
+    storage_   = Rhs::storage_
+  };
+  typedef typename Promote<typename Lhs::Type, typename Rhs::Type>::result_type Type;
+  typedef typename RemoveConst<Type>::Type ConstReturnType;  // not a reference as it is a temporary
+};
+
+} // namespace hidden
+
+/** @ingroup Arrays
+  * @brief Generic expression where a product operator is applied to two expressions
+  *
+  * @tparam Lhs left-hand side type
+  * @tparam Rhs right-hand side type
+  *
+  * This class represents an expression  where a product operator is applied to
+  * two expressions.
+  *
+  * It is the return type of product operator when the left hand side expression
+  * is a diagonal expression and the right and side expression is an array.
+  *
+  * Most of the time, this is the only way that it is used, so you typically
+  * don't have to name ArrayByDiagonalProduct types explicitly.
+  **/
 template<typename Lhs, typename Rhs>
-class DiagonalByArrayProduct : public ExprBase< DiagonalByArrayProduct<Lhs, Rhs> >
-                              , public TRef<1>
+class DiagonalByArrayProduct: public ExprBase< DiagonalByArrayProduct<Lhs, Rhs> >
+                            , public TRef<1>
 {
   public:
     typedef ExprBase< DiagonalByArrayProduct<Lhs, Rhs> > Base;
     typedef typename hidden::Traits<DiagonalByArrayProduct>::Type Type;
-    typedef typename hidden::Traits<DiagonalByArrayProduct>::ReturnType ReturnType;
+    typedef typename hidden::Traits<DiagonalByArrayProduct>::ConstReturnType ConstReturnType;
 
     enum
     {
@@ -526,14 +486,15 @@ class DiagonalByArrayProduct : public ExprBase< DiagonalByArrayProduct<Lhs, Rhs>
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return lhs_.rows();}
     /** @return the columns range */
-    inline ColRange const&colsImpl() const { return rhs_.cols();}
+    inline ColRange const& colsImpl() const { return rhs_.cols();}
 
-    /** access to the element (i,j) */
-    inline ReturnType elt2Impl(int i, int j) const { return lhs_.elt(i)*rhs_.elt(i,j);}
-    /** access to the ith element */
-    inline ReturnType elt1Impl(int i) const { return lhs_.elt(i)*rhs_.elt(i);}
-    /** access to the element */
-    inline ReturnType elt0Impl() const { return lhs_.elt()*rhs_.elt();}
+    /** @return element (i,j) */
+    inline ConstReturnType elt2Impl(int i, int j) const { return lhs_.elt(i)*rhs_.elt(i,j);}
+    /** @return ith element
+     *  @note make sense if rhs_ is a diagonal matrix, in this case this operator is also a
+     *  diagonal matrix.
+     **/
+    inline ConstReturnType elt1Impl(int i) const { return lhs_.elt(i)*rhs_.elt(i);}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
@@ -545,14 +506,117 @@ class DiagonalByArrayProduct : public ExprBase< DiagonalByArrayProduct<Lhs, Rhs>
     Rhs const& rhs_;
 };
 
+
+/** @ingroup Arrays
+  * @brief Generic expression where a product operator is applied to two expressions
+  *
+  * @tparam Lhs left-hand side type
+  * @tparam Rhs right-hand side type
+  *
+  * This class represents an expression  where a product operator is applied to
+  * two expressions.
+  *
+  * It is the return type of product operator when the left hand side expression
+  * is a (column) vector and the right and side expression is a point (a row vector).
+  *
+  * Most of the time, this is the only way that it is used, so you typically
+  * don't have to name VectorByPointProduct types explicitly.
+  **/
 template<typename Lhs, typename Rhs>
-class PointByArrayProduct : public ExprBase< PointByArrayProduct<Lhs, Rhs> >
-                           , public TRef<1>
+class VectorByPointProduct: public ExprBase< VectorByPointProduct<Lhs, Rhs> >
+                          , public TRef<1>
+{
+  public:
+    typedef ExprBase< VectorByPointProduct<Lhs, Rhs> > Base;
+    typedef typename hidden::Traits<VectorByPointProduct>::Type Type;
+    typedef typename hidden::Traits<VectorByPointProduct>::ConstReturnType ConstReturnType;
+
+    enum
+    {
+      structure_ = hidden::Traits<VectorByPointProduct>::structure_,
+      orient_    = hidden::Traits<VectorByPointProduct>::orient_,
+      sizeRows_  = hidden::Traits<VectorByPointProduct>::sizeRows_,
+      sizeCols_  = hidden::Traits<VectorByPointProduct>::sizeCols_,
+      storage_   = hidden::Traits<VectorByPointProduct>::storage_
+    };
+    /** Type of the Range for the rows */
+    typedef TRange<sizeRows_> RowRange;
+    /** Type of the Range for the columns */
+    typedef TRange<sizeCols_> ColRange;
+
+    /** constructor */
+    VectorByPointProduct( const Lhs& lhs, const Rhs& rhs): Base(), lhs_(lhs), rhs_(rhs){}
+    /**  @return the range of the rows */
+    inline RowRange const& rowsImpl() const { return lhs_.rows();}
+    /** @return the columns range */
+    inline ColRange const& colsImpl() const { return rhs_.cols();}
+
+    /** @return the element (i,j) */
+    inline ConstReturnType elt2Impl(int i, int j) const { return lhs_.elt(i)*rhs_.elt(j);}
+    /** @return the element */
+    inline ConstReturnType elt0Impl() const { return lhs_.elt()*rhs_.elt();}
+
+    /** @return the left hand side expression */
+    inline Lhs const& lhs() const { return lhs_; }
+    /** @return the right hand side expression */
+    inline Rhs const& rhs() const { return rhs_; }
+
+  protected:
+    Lhs const& lhs_;
+    Rhs const& rhs_;
+};
+
+
+
+namespace hidden
+{
+/** @ingroup hidden
+ *  @brief Traits class for the PointByArrayProduct
+ */
+template< typename Lhs, typename Rhs>
+struct Traits< PointByArrayProduct < Lhs, Rhs> >
+{
+  enum
+   {
+     structure_ = Arrays::point_,
+     sizeRows_  = 1,
+     sizeCols_  = Traits<Rhs>::sizeCols_,
+     orient_    = Arrays::by_row_,
+     storage_   = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
+                  ? int(Arrays::dense_) : int(Arrays::sparse_)
+   };
+
+  typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
+  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
+
+  typedef CAllocator<Type, sizeRows_, sizeCols_, orient_> Allocator;
+};
+
+} // namespace hidden
+
+/** @ingroup Arrays
+  * @brief Generic expression where a product operator is applied to two expressions
+  *
+  * @tparam Lhs left-hand side type
+  * @tparam Rhs right-hand side type
+  *
+  * This class represents an expression  where a product operator is applied to
+  * two expressions.
+  *
+  * It is the return type of product operator when the left hand side expression
+  * is a point (a  row vector) and the right and side expression is an array.
+  *
+  * Most of the time, this is the only way that it is used, so you typically
+  * don't have to name PointByArrayProduct types explicitly.
+  **/
+template<typename Lhs, typename Rhs>
+class PointByArrayProduct: public ExprBase< PointByArrayProduct<Lhs, Rhs> >
+                         , public TRef<1>
 {
   public:
     typedef ExprBase< PointByArrayProduct<Lhs, Rhs> > Base;
     typedef typename hidden::Traits<PointByArrayProduct>::Type Type;
-    typedef typename hidden::Traits<PointByArrayProduct>::ReturnType ReturnType;
+    typedef typename hidden::Traits<PointByArrayProduct>::ConstReturnType ConstReturnType;
     typedef typename hidden::Traits<PointByArrayProduct>::Allocator Allocator;
 
     enum
@@ -569,8 +633,8 @@ class PointByArrayProduct : public ExprBase< PointByArrayProduct<Lhs, Rhs> >
     typedef TRange<sizeCols_> ColRange;
 
     inline PointByArrayProduct( const Lhs& lhs, const Rhs& rhs)
-                              : Base(), lhs_(lhs), rhs_(rhs)
-                              , result_(1, rhs.sizeCols(), Type(0))
+                             : Base(), lhs_(lhs), rhs_(rhs)
+                             , result_(1, rhs.sizeCols(), Type(0))
     {
       if (lhs.range() != rhs.rows())
       { STKRUNTIME_ERROR_2ARG(PointByArrayProduct, lhs.range(), rhs.rows(), sizes mismatch);}
@@ -580,21 +644,21 @@ class PointByArrayProduct : public ExprBase< PointByArrayProduct<Lhs, Rhs> >
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return result_.rows();}
     /** @return the columns range */
-    inline ColRange const&colsImpl() const { return result_.cols();}
+    inline ColRange const& colsImpl() const { return result_.cols();}
 
     /** @return the element (i,j) */
-    inline ReturnType elt2Impl(int i, int j) const { return result_.elt(i, j);}
+    inline ConstReturnType elt2Impl(int i, int j) const { return result_.elt(i, j);}
     /** @return the ith element */
-    inline ReturnType elt1Impl(int i) const { return result_.elt(i);}
+    inline ConstReturnType elt1Impl(int i) const { return result_.elt(i);}
     /** @return the element */
-    inline ReturnType elt0Impl() const { return result_.elt();}
+    inline ConstReturnType elt0Impl() const { return result_.elt();}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
     /** @return the right hand side nested expression */
     inline Rhs const& rhs() const { return rhs_; }
     /** @return the right hand side nested expression */
-    Rhs const& result() const { return result_; }
+    Allocator const& result() const { return result_; }
 
   protected:
     Lhs const& lhs_;
@@ -604,16 +668,56 @@ class PointByArrayProduct : public ExprBase< PointByArrayProduct<Lhs, Rhs> >
     Allocator result_;
 };
 
+namespace hidden
+{
+/** @ingroup hidden
+ *  @brief Traits class for the ArrayByVectorProduct class
+ */
+template< typename Lhs, typename Rhs>
+struct Traits< ArrayByVectorProduct < Lhs, Rhs> >
+{
+  enum
+   {
+     structure_ = Arrays::vector_,
+     orient_    = Arrays::by_col_,
+     sizeRows_  = Traits<Lhs>::sizeRows_,
+     sizeCols_  = 1,
+     storage_   = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
+                  ? int(Arrays::dense_) : int(Arrays::sparse_)
+   };
 
+  typedef ProductTraits<Lhs, Rhs, Lhs::structure_, Rhs::structure_> Base;
+  typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
+  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
+  typedef CAllocator<Type, sizeRows_, sizeCols_, orient_> Allocator;
+};
+
+} // nmaespace hidden
+
+/** @ingroup Arrays
+  * @brief Generic expression where a product operator is applied to two expressions
+  *
+  * @tparam Lhs left-hand side type
+  * @tparam Rhs right-hand side type
+  *
+  * This class represents an expression  where a product operator is applied to
+  * two expressions.
+  *
+  * It is the return type of product operator when the left hand side expression
+  * is an array and the right and side expression is a (column) vector.
+  *
+  * Most of the time, this is the only way that it is used, so you typically
+  * don't have to name ArrayByVectorProduct types explicitly.
+  **/
 template<typename Lhs, typename Rhs>
-class ArrayByVectorProduct : public ExprBase< ArrayByVectorProduct<Lhs, Rhs> >
+class ArrayByVectorProduct: public ExprBase< ArrayByVectorProduct<Lhs, Rhs> >
                            , public TRef<1>
 {
   public:
     typedef ExprBase< ArrayByVectorProduct<Lhs, Rhs> > Base;
     typedef typename hidden::Traits<ArrayByVectorProduct>::Type Type;
-    typedef typename hidden::Traits<ArrayByVectorProduct>::ReturnType ReturnType;
+    typedef typename hidden::Traits<ArrayByVectorProduct>::ConstReturnType ConstReturnType;
     typedef typename hidden::Traits<ArrayByVectorProduct>::Allocator Allocator;
     enum
     {
@@ -629,8 +733,8 @@ class ArrayByVectorProduct : public ExprBase< ArrayByVectorProduct<Lhs, Rhs> >
     typedef TRange<sizeCols_> ColRange;
 
     ArrayByVectorProduct( const Lhs& lhs, const Rhs& rhs)
-                              : Base(), lhs_(lhs), rhs_(rhs)
-                              , result_(lhs.sizeRows(), 1, Type(0))
+                        : Base(), lhs_(lhs), rhs_(rhs)
+                        , result_(lhs.sizeRows(), 1, Type(0)) // create result
     {
       if (lhs.cols() != rhs.range())
       { STKRUNTIME_ERROR_NO_ARG(ArrayByVectorProduct, sizes mismatch);}
@@ -640,14 +744,14 @@ class ArrayByVectorProduct : public ExprBase< ArrayByVectorProduct<Lhs, Rhs> >
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return result_.rows();}
     /** @return the columns range */
-    inline ColRange const&colsImpl() const { return result_.cols();}
+    inline ColRange const& colsImpl() const { return result_.cols();}
 
     /** @return the element (i,j) */
-    inline ReturnType elt2Impl(int i, int j) const { return result_.elt(i, j);}
+    inline ConstReturnType elt2Impl(int i, int j) const { return result_.elt(i, j);}
     /** @return the ith element */
-    inline ReturnType elt1Impl(int i) const { return result_.elt(i);}
+    inline ConstReturnType elt1Impl(int i) const { return result_.elt(i);}
     /** @return the element */
-    inline ReturnType elt0Impl() const { return result_.elt();}
+    inline ConstReturnType elt0Impl() const { return result_.elt();}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
@@ -664,60 +768,64 @@ class ArrayByVectorProduct : public ExprBase< ArrayByVectorProduct<Lhs, Rhs> >
     Allocator result_;
 };
 
-template<typename Lhs, typename Rhs>
-class VectorByPointProduct : public ExprBase< VectorByPointProduct<Lhs, Rhs> >
-                           , public TRef<1>
+namespace hidden
 {
-  public:
-    typedef ExprBase< VectorByPointProduct<Lhs, Rhs> > Base;
-    typedef typename hidden::Traits<VectorByPointProduct>::Type Type;
-    typedef typename hidden::Traits<VectorByPointProduct>::ReturnType ReturnType;
+/** @ingroup hidden
+ *  @brief Traits class for the ArrayByArrayProduct class
+ */
+template< typename Lhs, typename Rhs>
+struct Traits< ArrayByArrayProduct<Lhs, Rhs> >
+{
+  typedef ProductTraits<Lhs, Rhs, Traits<Lhs>::structure_, Traits<Rhs>::structure_> Base;
 
-    enum
-    {
-      structure_ = hidden::Traits<VectorByPointProduct>::structure_,
-      orient_    = hidden::Traits<VectorByPointProduct>::orient_,
-      sizeRows_  = hidden::Traits<VectorByPointProduct>::sizeRows_,
-      sizeCols_  = hidden::Traits<VectorByPointProduct>::sizeCols_,
-      storage_   = hidden::Traits<VectorByPointProduct>::storage_
-    };
-    /** Type of the Range for the rows */
-    typedef TRange<sizeRows_> RowRange;
-    /** Type of the Range for the columns */
-    typedef TRange<sizeCols_> ColRange;
+  enum
+  {
+    structure_ = Base::structure_,
+    sizeRows_ = Traits<Lhs>::sizeRows_,
+    sizeCols_ = Traits<Rhs>::sizeCols_,
+    // if there is more block on the left side, we use bp --> result is by_col_
+    orient_   =  ( Traits<Lhs>::sizeRows_ == UnknownSize || Traits<Rhs>::sizeCols_ == UnknownSize)
+                 ? Traits<Rhs>::orient_
+                 : (Traits<Lhs>::sizeRows_)/blockSize_ < (Traits<Rhs>::sizeCols_)/blockSize_
+                   ? int(Arrays::by_row_) : int(Arrays::by_col_),
+    storage_  = ( Traits<Lhs>::storage_ == int(Arrays::dense_)) || (Traits<Rhs>::storage_ == int(Arrays::dense_))
+                  ? int(Arrays::dense_) : int(Arrays::sparse_)
+  };
+  typedef typename hidden::Promote< typename Lhs::Type, typename Rhs::Type>::result_type Type;
+  typedef typename RemoveConst<Type>::Type const& ConstReturnType;
 
-    VectorByPointProduct( const Lhs& lhs, const Rhs& rhs)
-                               : Base(), lhs_(lhs), rhs_(rhs)
-    {}
-    /**  @return the range of the rows */
-    inline RowRange const& rowsImpl() const { return lhs_.rows();}
-    /** @return the columns range */
-    inline ColRange const&colsImpl() const { return rhs_.cols();}
-
-    /** @return the element (i,j) */
-    inline ReturnType elt2Impl(int i, int j) const { return lhs_.elt(i)*rhs_.elt(j);}
-    /** @return the element */
-    inline ReturnType elt0Impl() const { return lhs_.elt()*rhs_.elt();}
-
-    /** @return the left hand side expression */
-    inline Lhs const& lhs() const { return lhs_; }
-    /** @return the right hand side expression */
-    inline Rhs const& rhs() const { return rhs_; }
-
-  protected:
-    Lhs const& lhs_;
-    Rhs const& rhs_;
+  typedef typename Base::Allocator Allocator;
 };
+
+} // namespace hidden
+
+
+
 // forward declaration
 template< typename Lhs, typename Rhs> class ArrayByArrayProductBase;
 
+/** @ingroup Arrays
+  * @brief Generic expression where a product operator is applied to two expressions
+  *
+  * @tparam Lhs left-hand side type
+  * @tparam Rhs right-hand side type
+  *
+  * This class represents an expression  where a product operator is applied to
+  * two expressions.
+  *
+  * It is the return type of product operator when the left hand side expression
+  * is an array and the right and side expression is an array.
+  *
+  * Most of the time, this is the only way that it is used, so you typically
+  * don't have to name ArrayByArrayProduct types explicitly.
+  **/
 template<typename Lhs, typename Rhs>
-class ArrayByArrayProduct : public ArrayByArrayProductBase< Lhs, Rhs >, public TRef<1>
+class ArrayByArrayProduct: public ArrayByArrayProductBase< Lhs, Rhs >, public TRef<1>
 {
   public:
     typedef ArrayByArrayProductBase< Lhs, Rhs> Base;
     typedef typename hidden::Traits<ArrayByArrayProduct>::Type Type;
-    typedef typename hidden::Traits<ArrayByArrayProduct>::ReturnType ReturnType;
+    typedef typename hidden::Traits<ArrayByArrayProduct>::ConstReturnType ConstReturnType;
 
     typedef typename hidden::Traits<ArrayByArrayProduct < Lhs, Rhs> >::Allocator Allocator;
 
@@ -757,7 +865,7 @@ class ArrayByArrayProduct : public ArrayByArrayProductBase< Lhs, Rhs >, public T
     /**  @return the range of the rows */
     inline RowRange const& rowsImpl() const { return lhs_.rows();}
     /** @return the columns range */
-    inline ColRange const&colsImpl() const { return rhs_.cols();}
+    inline ColRange const& colsImpl() const { return rhs_.cols();}
 
     /** @return the left hand side expression */
     inline Lhs const& lhs() const { return lhs_; }
@@ -778,26 +886,23 @@ class ArrayByArrayProduct : public ArrayByArrayProductBase< Lhs, Rhs >, public T
   * @brief implement the access to the elements in the (2D) general case.
   **/
 template< typename Lhs, typename Rhs>
-class ArrayByArrayProductBase : public ExprBase< ArrayByArrayProduct<Lhs, Rhs> >
+class ArrayByArrayProductBase: public ExprBase< ArrayByArrayProduct<Lhs, Rhs> >
 {
   public:
     typedef ArrayByArrayProduct<Lhs, Rhs> Derived;
     typedef ExprBase< Derived> Base;
 
     typedef typename hidden::Traits<Derived>::Type Type;
-    typedef typename hidden::Traits<Derived>::ReturnType ReturnType;
+    typedef typename hidden::Traits<Derived>::ConstReturnType ConstReturnType;
 
     /** constructor. */
-    inline ArrayByArrayProductBase() : Base() {}
-    /** access to the element (i,j) */
-    inline Type const& elt2Impl(int i, int j) const
-    { return this->asDerived().result().elt(i,j);}
-    /** access to the element i */
-    inline Type const& elt1Impl(int i) const
-    { return this->asDerived().result().elt(i);}
-    /** access to the element */
-    inline Type const& elt0Impl() const
-    { return this->asDerived().result().elt();}
+    inline ArrayByArrayProductBase(): Base() {}
+    /** access to element (i,j) */
+    inline Type const& elt2Impl(int i, int j) const { return this->asDerived().result().elt(i,j);}
+    /** access to ith element */
+    inline Type const& elt1Impl(int i) const  { return this->asDerived().result().elt(i);}
+    /** access to number */
+    inline Type const& elt0Impl() const { return this->asDerived().result().elt();}
 };
 
 }  // namespace STK
