@@ -54,7 +54,7 @@ namespace hidden
 {
 
 /** @ingroup hidden
- *  @brief Specialization of the Traits class for the  STK::IntegerVector class.
+ *  @brief Specialization of the Traits class for the  STK::RVector class.
  **/
 template<typename Type_>
 struct Traits< RVector<Type_> >
@@ -66,7 +66,7 @@ struct Traits< RVector<Type_> >
     typedef ColOperator< RVector<Type_> > Col;
     typedef RowOperator< RMatrix<Type_> > SubRow;
     typedef ColOperator< RMatrix<Type_> > SubCol;
-    typedef SubOperator< RMatrix<Type_> > SubVector;
+    typedef SubVectorOperator< RMatrix<Type_>, UnknownSize > SubVector;
     typedef Void SubArray;
     typedef Void Number;
 
@@ -79,7 +79,7 @@ struct Traits< RVector<Type_> >
       structure_ = Arrays::vector_,
       orient_    = Arrays::by_col_,
       sizeRows_  = UnknownSize,
-      sizeCols_  = UnknownSize,
+      sizeCols_  = 1,
       storage_   = Arrays::dense_
     };
 };
@@ -87,15 +87,9 @@ struct Traits< RVector<Type_> >
 } // namespace hidden
 
 template <typename Type_>
-class RVector : public ArrayBase< RVector<Type_> >, public TRef<1>
+class RVector: public ArrayBase< RVector<Type_> >, public TRef<1>
 {
   public:
-    typedef typename hidden::Traits<RVector<Type_> >::Row Row;
-    typedef typename hidden::Traits<RVector<Type_> >::Col Col;
-    typedef typename hidden::Traits<RVector<Type_> >::SubRow SubRow;
-    typedef typename hidden::Traits<RVector<Type_> >::SubCol SubCol;
-    typedef typename hidden::Traits<RVector<Type_> >::SubVector SubVector;
-
     typedef typename hidden::Traits<RVector<Type_> >::Type Type;
     typedef typename hidden::Traits<RVector<Type_> >::ReturnType ReturnType;
     enum
@@ -154,12 +148,6 @@ class RVector : public ArrayBase< RVector<Type_> >, public TRef<1>
     /** @return the number of columns */
     inline int sizeCols() const { return 1;}
 
-    /** @return the j-th column of this. */
-    inline Col colImpl(int j) const { return Col(this->asDerived(), j);}
-    /** @return the i-th row of this. */
-    inline Row rowImpl(int i) const { return Row(this->asDerived(), i);}
-    /** @return the i-th row of this. */
-    inline SubVector sub(Range I) const { return SubVector(this->asDerived(), I);}
     /** @return a constant reference on ith element
      *  @param i index of the ith element
      **/
@@ -176,7 +164,7 @@ class RVector : public ArrayBase< RVector<Type_> >, public TRef<1>
     /** @return a reference on the element (i,j)
      *  @param i, j indexes of the row and of the column
      **/
-    inline Type& elt2Impl(int i, int j) { return (vector_[i]);}
+    inline Type& elt2Impl(int i, int j) { return static_cast<Type&>(vector_[i]);}
     /** overwrite the RVector with vec using Rcpp::operator=.
      *  @param vec the vector to copy
      **/
@@ -206,7 +194,7 @@ class RVector : public ArrayBase< RVector<Type_> >, public TRef<1>
   private:
     Rcpp::Vector<Rtype_> vector_;
     RowRange rows_;
-    const ColRange cols_;
+    ColRange cols_;
 };
 
 } // namespace STK
