@@ -96,19 +96,17 @@ struct TestEndOfLineOp
   typedef bool result_type;
   typedef Char param1_type ;
   inline TestEndOfLineOp(char* c): last_(c) {}
-  Char* last_;
-  inline result_type operator()(Char c)
-  {
-    *last_ = c;
-    return (c == _T('\n'));
-  }
+
+  inline result_type operator()(Char c) { *last_ = c; return (c == _T('\n'));}
+  private:
+    Char* last_;
 };
 
 //------------------------------------------------------------------------------
 // binary functors:
 //------------------------------------------------------------------------------
 /** @ingroup Functors
-  * @brief Template functor testing if a number is equal to an other
+  * @brief Template functor testing if a number is equal to an other number
   */
 template<class Type1, class Type2 = Type1>
 struct EqualOp
@@ -122,7 +120,7 @@ struct EqualOp
   { return value1 == value2;}
 };
 /** @ingroup Functors
-  * @brief Template functor testing if a number is not equal to an other
+  * @brief Template functor testing if a number is not equal to an other number
   */
 template<class Type1, class Type2 = Type1>
 struct NotEqualOp
@@ -136,7 +134,7 @@ struct NotEqualOp
   { return value1 != value2;}
 };
 /** @ingroup Functors
-  * @brief Template functor testing if a number is less than an other
+  * @brief Template functor testing if a number is less than an other number
   */
 template<class Type1, class Type2 = Type1>
 struct LessOp
@@ -150,21 +148,7 @@ struct LessOp
   { return value1 < value2;}
 };
 /** @ingroup Functors
-  * @brief Template functor testing if a number is less or equal than an other
-  */
-template<class Type1, class Type2 = Type1>
-struct LeqOp
-{
-  enum { NbParam_ = 2 };
-  typedef bool result_type;
-  typedef typename hidden::RemoveConst<Type1>::Type param1_type ;
-  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
-
-  inline result_type operator()(param1_type const& value1, param2_type value2) const
-  { return value1 <= value2;}
-};
-/** @ingroup Functors
-  * @brief Template functor testing if a number is greater than an other
+  * @brief Template functor testing if a number is greater than an other number
   */
 template<class Type1, class Type2 = Type1>
 struct GreaterOp
@@ -178,7 +162,21 @@ struct GreaterOp
   { return value1 > value2;}
 };
 /** @ingroup Functors
-  * @brief Template functor testing if a number is greater or equal than an other
+  * @brief Template functor testing if a number is less or equal than an other number
+  */
+template<class Type1, class Type2 = Type1>
+struct LeqOp
+{
+  enum { NbParam_ = 2 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type1>::Type param1_type ;
+  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
+
+  inline result_type operator()(param1_type const& value1, param2_type value2) const
+  { return value1 <= value2;}
+};
+/** @ingroup Functors
+  * @brief Template functor testing if a number is greater or equal than an other number
   */
 template<class Type1, class Type2 = Type1>
 struct GeqOp
@@ -191,19 +189,71 @@ struct GeqOp
   inline result_type operator()(param1_type const& value1, param2_type value2) const
   { return value1 >= value2;}
 };
+
 /** @ingroup Functors
-  * @brief Template functor which compute the maximum of two numbers
+  * @brief Template functor which compute the sum of two numbers
   */
 template<class Type1, class Type2 = Type1>
-struct MaxOp
+struct SumOp
 {
   enum { NbParam_ = 2 };
   typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
-  typedef typename hidden::RemoveConst<Type1>::Type param1_type ;
+  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
   typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
 
-  inline result_type operator()(param1_type const&  value1, param2_type value2) const
-  { return  (value1 < value2) ? value2: value1 ;}
+  inline result_type operator()(param1_type a, param2_type b) const { return a + b;}
+};
+/** @ingroup Functors
+  * @brief Template functor which compute the difference of two numbers
+  */
+template<class Type1, class Type2 = Type1>
+struct DifferenceOp
+{
+  enum { NbParam_ = 2 };
+  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
+  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
+
+  inline result_type operator()(param1_type a, param2_type b) const { return a - b;}
+};
+/** @ingroup Functors
+  * @brief Template functor which compute the product of two numbers
+  */
+template<class Type1, class Type2 = Type1>
+struct ProductOp
+{
+  enum { NbParam_ = 2 };
+  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
+  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
+
+  inline result_type operator()(param1_type a, param2_type b) const { return a * b;}
+};
+/** @ingroup Functors
+  * @brief Template functor which compute the division of two numbers
+  */
+template<class Type1, class Type2 = Type1>
+struct DivisionOp
+{
+  enum { NbParam_ = 2 };
+  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
+  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
+
+  inline result_type operator()(param1_type a, param2_type b) const { return a / b;}
+};
+/** @ingroup Functors
+  * @brief Template functor which compute the modulo of two numbers
+  */
+template<class Type1, class Type2 = Type1>
+struct ModuloOp
+{
+  enum { NbParam_ = 2 };
+  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
+  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
+
+  inline result_type operator()(param1_type a, param2_type b) const { return a % b;}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the minimum of two numbers
@@ -220,158 +270,88 @@ struct MinOp
   { return  (value1 < value2) ? value1: value2 ;}
 };
 /** @ingroup Functors
-  * @brief Template functor which compute the sum of two numbers
+  * @brief Template functor which compute the maximum of two numbers
   */
 template<class Type1, class Type2 = Type1>
-struct SumOp
+struct MaxOp
 {
   enum { NbParam_ = 2 };
   typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
-  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
+  typedef typename hidden::RemoveConst<Type1>::Type param1_type ;
   typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
 
-  inline result_type operator()(param1_type a, param2_type b) const
-  { return a + b; }
+  inline result_type operator()(param1_type const&  value1, param2_type value2) const
+  { return  (value1 < value2) ? value2: value1 ;}
 };
 /** @ingroup Functors
-  * @brief Template functor which compute the product of two numbers
+  * @brief Template functor which compute the logical and of two numbers
   */
 template<class Type1, class Type2 = Type1>
-struct ProductOp
+struct LogicalAndOp
 {
   enum { NbParam_ = 2 };
-  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef bool result_type;
   typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
   typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
 
-  inline result_type operator()(param1_type a, param2_type b) const
-  { return a * b; }
+  inline result_type operator()(param1_type a, param2_type b) const { return a && b;}
 };
 /** @ingroup Functors
-  * @brief Template functor which compute the difference of two numbers
+  * @brief Template functor which compute the logical or of two numbers
   */
 template<class Type1, class Type2 = Type1>
-struct DifferenceOp
+struct LogicalOrOp
 {
   enum { NbParam_ = 2 };
-  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef bool result_type;
   typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
   typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
 
-  inline result_type operator()(param1_type a, param2_type b) const
-  { return a - b; }
+  inline result_type operator()(param1_type a, param2_type b) const { return a || b;}
 };
 /** @ingroup Functors
-  * @brief Template functor which compute the division of two numbers
+  * @brief Template functor which compute the bitwise and of two numbers
   */
 template<class Type1, class Type2 = Type1>
-struct DivOp
+struct BitwiseAndOp
 {
   enum { NbParam_ = 2 };
   typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
   typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
   typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
 
-  inline result_type operator()(param1_type a, param2_type b) const
-  { return a / b; }
+  inline result_type operator()(param1_type a, param2_type b) const { return a & b;}
+};
+/** @ingroup Functors
+  * @brief Template functor which compute the bitwise or of two numbers
+  */
+template<class Type1, class Type2 = Type1>
+struct BitwiseOrOp
+{
+  enum { NbParam_ = 2 };
+  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
+  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
+
+  inline result_type operator()(param1_type a, param2_type b) const { return a | b;}
+};
+/** @ingroup Functors
+  * @brief Template functor which compute the bitwise xor of two numbers
+  */
+template<class Type1, class Type2 = Type1>
+struct BitwiseXorOp
+{
+  enum { NbParam_ = 2 };
+  typedef typename hidden::Promote<Type1, Type2>::result_type result_type;
+  typedef typename hidden::RemoveConst<Type1>::Type const& param1_type ;
+  typedef typename hidden::RemoveConst<Type2>::Type const& param2_type ;
+
+  inline result_type operator()(param1_type a, param2_type b) const { return a ^ b;}
 };
 
 //------------------------------------------------------------------------------
 // unary functors:
 //------------------------------------------------------------------------------
-/** @ingroup Functors
-  * @brief Template functor to test if a number is less than a fixed number
-  */
-template<typename Type>
-struct LessThanOp
-{
-  enum { NbParam_ = 1 };
-  typedef bool result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type;
-
-  inline LessThanOp(const LessThanOp& other): other_(other.other_) { }
-  inline LessThanOp(Type other): other_(other) { }
-  inline result_type operator() (param1_type const& a) const { return a < other_; }
-  const Type other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to test if a number is less or equal than a fixed number
-  */
-template<typename Type>
-struct LeqThanOp
-{
-  enum { NbParam_ = 1 };
-  typedef bool result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline LeqThanOp(LeqThanOp const& other): other_(other.other_)
-  {}
-  inline LeqThanOp(Type other): other_(other)
-  {}
-  inline result_type operator() (param1_type const& a) const
-  { return (a <= other_); }
-  const Type other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to test if a number is greater than a fixed number
-  */
-template<typename Type>
-struct GreaterThanOp
-{
-  enum { NbParam_ = 1 };
-  typedef bool result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline GreaterThanOp(const GreaterThanOp& other): other_(other.other_) { }
-  inline GreaterThanOp(Type other): other_(other) { }
-  inline result_type operator() (param1_type const& a) const { return a > other_; }
-  const Type other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to test if a number is greater or equal than a fixed number
-  */
-template<typename Type>
-struct GeqThanOp
-{
-  enum { NbParam_ = 1 };
-  typedef bool result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline GeqThanOp(const GeqThanOp& other): other_(other.other_) { }
-  inline GeqThanOp(Type other): other_(other) { }
-  inline result_type operator() (param1_type const& a) const { return a >= other_; }
-  const Type other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to test if a number is equal to a fixed number
-  */
-template<typename Type>
-struct EqualThanOp
-{
-  enum { NbParam_ = 1 };
-  typedef bool result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline EqualThanOp(const EqualThanOp& other): other_(other.other_) { }
-  inline EqualThanOp(Type other): other_(other) {}
-  inline result_type operator() (param1_type const& a) const { return a == other_; }
-  const Type other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to test if a number is different than a fixed number
-  */
-template<typename Type>
-struct NotEqualThanOp
-{
-  enum { NbParam_ = 1 };
-  typedef bool result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline NotEqualThanOp(const NotEqualThanOp& other): other_(other.other_) { }
-  inline NotEqualThanOp(Type other): other_(other) { }
-  inline result_type operator() (param1_type const& a) const { return a != other_; }
-  const Type other_;
-};
 /** @ingroup Functors
   * @brief Template functor testing if a number is a NA value
   */
@@ -382,11 +362,11 @@ struct IsNaOp
   typedef bool result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator()(param1_type const&  value1) const
-  { return Arithmetic<param1_type>::isNA(value1);}
+  inline result_type operator()(param1_type const&  value) const
+  { return Arithmetic<param1_type>::isNA(value);}
 };
 /** @ingroup Functors
-  * @brief Template functor testing if a number is a NA value
+  * @brief Template functor giving the not value
   */
 template<class Type>
 struct NegOp
@@ -395,8 +375,7 @@ struct NegOp
   typedef bool result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator()(param1_type const& value1) const
-  { return !value1;}
+  inline result_type operator()(param1_type const& value) const  { return !value;}
 };
 /** @ingroup Functors
   * @brief Template functor testing if a number is a finite value
@@ -408,8 +387,8 @@ struct IsFiniteOp
   typedef bool result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator()(param1_type const& value1) const
-  { return Arithmetic<param1_type>::isFinite(value1);}
+  inline result_type operator()(param1_type const& value) const
+  { return Arithmetic<param1_type>::isFinite(value);}
 };
 /** @ingroup Functors
   * @brief Template functor testing if a number is an infinite value
@@ -424,6 +403,359 @@ struct IsInfiniteOp
   inline result_type operator()(param1_type const& value1) const
   { return Arithmetic<param1_type>::isInfinite(value1);}
 };
+/** @ingroup Functors
+  * @brief Template functor to test if a number is equal to a fixed value
+  */
+template<typename Type>
+struct EqualWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline EqualWithOp(EqualWithOp const& value): value_(value.value_) {}
+  inline EqualWithOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a == value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to test if a number is different than a fixed value
+  */
+template<typename Type>
+struct NotEqualWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline NotEqualWithOp(NotEqualWithOp const& value): value_(value.value_) {}
+  inline NotEqualWithOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a != value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to test if a number is less than a fixed value
+  */
+template<typename Type>
+struct LessThanOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type;
+
+  inline LessThanOp(LessThanOp const& value): value_(value.value_) {}
+  inline LessThanOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a < value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to test if a number is greater than a fixed value
+  */
+template<typename Type>
+struct GreaterThanOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline GreaterThanOp(GreaterThanOp const& value): value_(value.value_) {}
+  inline GreaterThanOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a > value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to test if a number is less or equal than a fixed value
+  */
+template<typename Type>
+struct LeqThanOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline LeqThanOp(LeqThanOp const& value): value_(value.value_) {}
+  inline LeqThanOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return (a <= value_);}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to test if a number is greater or equal than a fixed value
+  */
+template<typename Type>
+struct GeqThanOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline GeqThanOp(GeqThanOp const& value): value_(value.value_) {}
+  inline GeqThanOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a >= value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to add a number to a fixed value
+  */
+template<typename Type>
+struct SumWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline SumWithOp(SumWithOp const& value): value_(value.value_) {}
+  inline SumWithOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a + value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to add a number to a fixed value
+  */
+template<typename Type>
+struct DifferenceWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline DifferenceWithOp(DifferenceWithOp const& value): value_(value.value_) {}
+  inline DifferenceWithOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a - value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor computing the product of a number by a fixed value
+  */
+template<typename Type>
+struct ProductWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline ProductWithOp(Type value): value_(value) {}
+  inline ProductWithOp(ProductWithOp const& value): value_(value.value_) {}
+
+  inline result_type operator() (param1_type const& a) const {return a * value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Specialization for boolean
+  */
+template<>
+struct ProductWithOp<bool>
+{
+  enum { NbParam_ = 1 };
+  typedef bool const result_type;
+  typedef bool const param1_type;
+
+  inline ProductWithOp(bool value): value_(value) {}
+  inline ProductWithOp( ProductWithOp const& value): value_(value.value_) {}
+
+  inline result_type operator() (param1_type const& a) const {return a && value_;}
+  private:
+    bool const value_;
+};
+/** @ingroup Functors
+ *  base class for DivisionWithOp functors
+ **/
+template<typename Type, bool IsInt>
+struct DivisionWithBaseOp;
+
+template<typename Type>
+struct DivisionWithBaseOp<Type, false>
+{
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+  inline DivisionWithBaseOp(const DivisionWithBaseOp& value): value_(value.value_) {}
+  inline DivisionWithBaseOp(param1_type const& value): value_( static_cast<Type>(1) / value) {}
+
+  inline Type const operator() (param1_type const& a) const { return a * value_;}
+  private:
+    param1_type const value_;
+};
+
+template<typename Type>
+struct DivisionWithBaseOp<Type, true>
+{
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+  inline DivisionWithBaseOp( DivisionWithBaseOp const& value): value_(value.value_) {}
+  inline DivisionWithBaseOp( param1_type value): value_(value) {}
+
+  inline Type const operator()( param1_type a) const { return a / value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to divide a number by a fixed value
+  *
+  * This functor is used to implement the quotient of a matrix by
+  * a number where the number type is not necessarily a floating point type.
+  */
+template<typename Type>
+struct DivisionWithOp: public DivisionWithBaseOp<Type, hidden::IsInt<Type>::yes >
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline DivisionWithOp(param1_type const& value): DivisionWithBaseOp<Type, hidden::IsInt<Type>::yes >(value) {}
+};
+/** @ingroup Functors
+  * @brief Template functor to compute the minimum with a fixed value
+  */
+template<typename Type>
+struct ModuloWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline ModuloWithOp(Type const& value): value_(value) {}
+  inline ModuloWithOp( ModuloWithOp const& value): value_(value.value_) {}
+
+  inline result_type operator() (param1_type const& a) const {return a % value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to compute the minimum between a fixed value
+  */
+template<typename Type>
+struct MinWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline MinWithOp(Type const& value): value_(value) {}
+  inline MinWithOp( MinWithOp const& value): value_(value.value_) {}
+
+  inline result_type operator() (param1_type const& a) const {return std::min(a,value_);}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to compute the minimum with a fixed value
+  */
+template<typename Type>
+struct MaxWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline MaxWithOp(Type const& value): value_(value) {}
+  inline MaxWithOp( MaxWithOp const& value): value_(value.value_) {}
+
+  inline result_type operator() (param1_type const& a) const {return std::max(a,value_);}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor computing logical and with a fixed value
+  */
+template<typename Type>
+struct LogicalAndWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline LogicalAndWithOp(LogicalAndWithOp const& value): value_(value.value_) {}
+  inline LogicalAndWithOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a && value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor computing logical and with a fixed value
+  */
+template<typename Type>
+struct LogicalOrWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef bool result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline LogicalOrWithOp(LogicalOrWithOp const& value): value_(value.value_) {}
+  inline LogicalOrWithOp(Type value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const { return a || value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to compute the bitwise and with a fixed value
+  */
+template<typename Type>
+struct BitwiseAndWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline BitwiseAndWithOp(BitwiseAndWithOp const& value): value_(value.value_) {}
+  inline BitwiseAndWithOp(Type const& value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const {return a & value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to compute the bitwise or with a fixed value
+  */
+template<typename Type>
+struct BitwiseOrWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline BitwiseOrWithOp(BitwiseOrWithOp const& value): value_(value.value_) {}
+  inline BitwiseOrWithOp(Type const& value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const {return a | value_;}
+  private:
+    param1_type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor to compute the bitwise xor with a fixed value
+  */
+template<typename Type>
+struct BitwiseXorWithOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline BitwiseXorWithOp(BitwiseXorWithOp const& value): value_(value.value_) {}
+  inline BitwiseXorWithOp(Type const& value): value_(value) {}
+
+  inline result_type operator() (param1_type const& a) const {return a ^ value_;}
+  private:
+    param1_type const value_;
+};
+
+
+// Functors without value_
 /** @ingroup Functors
   * @brief Template functor which compute the opposite of a number
   */
@@ -434,10 +766,45 @@ struct OppositeOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator()(param1_type const& a) const
-  { return -a; }
+  inline result_type operator()(param1_type const& a) const { return -a;}
 };
+/** @ingroup Functors
+  * @brief Template functor which return a default value if the value is NA
+  */
+template<class Type>
+struct SafeOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
+  inline SafeOp(Type const value = Type()): value_(value) {}
+  inline SafeOp( SafeOp const& value): value_(value.value_) {}
+
+  inline result_type operator()(param1_type const& a) const
+  { return Arithmetic<Type>::isFinite(a) ? a: value_;}
+  private:
+    Type const value_;
+};
+/** @ingroup Functors
+  * @brief Template functor which compute safely the inverse of a number.
+  * If the number is 0, return 0.
+  */
+template<typename Type>
+struct SafeInverseOp
+{
+  enum { NbParam_ = 1 };
+  typedef Type result_type;
+  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
+
+  inline SafeInverseOp(Type const& tol = Arithmetic<Type>::epsilon()): tol_(tol) {}
+  inline SafeInverseOp( SafeInverseOp const& value): tol_(value.tol_) {}
+
+  inline result_type operator()(param1_type const& a) const
+  { return (std::abs(a)>tol_) ? Type(1)/a: Type(0);}
+  private:
+    Type const tol_;
+};
 /** @ingroup Functors
   * @brief Template functor which compute the absolute value of a number
   */
@@ -448,8 +815,7 @@ struct AbsOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator()(param1_type const& a) const
-  { return std::abs(a); }
+  inline result_type operator()(param1_type const& a) const { return std::abs(a);}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the exponential of a number
@@ -475,176 +841,6 @@ struct LogOp
 
   inline result_type operator()(param1_type const& a) const {return std::log(a);}
 };
-/** @ingroup Functors
-  * @brief Template functor to compute the minimum between a number and a fixed
-  * other one
-  */
-template<typename Type>
-struct MinimumOp
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef Type const param1_type ;
-
-  inline MinimumOp(Type const other): other_(other) {}
-  inline MinimumOp( MinimumOp const& other): other_(other.other_) {}
-  inline result_type operator() (param1_type const& a) const
-  {return std::min(a,other_); }
-  Type const other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to compute the minimum between a number and a fixed
-  * other one
-  */
-template<typename Type>
-struct MaximumOp
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef Type const param1_type ;
-
-  inline MaximumOp(Type const other): other_(other) {}
-  inline MaximumOp( MaximumOp const& other): other_(other.other_) {}
-  inline result_type operator() (param1_type const& a) const
-  {return std::max(a,other_); }
-  Type const other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to multiply a number by a fixed other one
-  */
-template<typename Type>
-struct MultipleOp
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef Type const param1_type ;
-
-  inline MultipleOp(Type const other): other_(other) {}
-  inline MultipleOp( MultipleOp const& other): other_(other.other_) {}
-  inline result_type operator() (param1_type const& a) const
-  {return a * other_; }
-  Type const other_;
-};
-/** @ingroup Functors
-  * @brief Specialization for boolean
-  */
-template<>
-struct MultipleOp<bool>
-{
-  enum { NbParam_ = 1 };
-  typedef bool const result_type;
-  typedef bool const param1_type ;
-
-  inline MultipleOp(bool const other): other_(other) {}
-  inline MultipleOp( MultipleOp const& other): other_(other.other_) {}
-  inline result_type operator() (param1_type const& a) const
-  {return a && other_; }
-  bool const other_;
-};
-
-
-/** @ingroup Functors
-  * @brief Template functor which return a default value if the value is NA
-  */
-template<class Type>
-struct SafeOp
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline SafeOp(Type const other = Type()): other_(other) {}
-  inline SafeOp( SafeOp const& other): other_(other.other_) {}
-  inline result_type operator()(param1_type const& a) const
-  { return Arithmetic<Type>::isFinite(a) ? a: other_; }
-  Type const other_;
-};
-/** @ingroup Functors
-  * @brief Template functor which compute safely the inverse of a number
-  */
-template<typename Type>
-struct SafeInverseOp
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline SafeInverseOp(Type const tol = Arithmetic<Type>::epsilon()): tol_(tol) {}
-  inline SafeInverseOp( SafeInverseOp const& other): tol_(other.tol_) {}
-  inline result_type operator()(param1_type const& a) const
-  { return (std::abs(a)>tol_) ? Type(1)/a: 1.; }
-  Type const tol_;
-};
-
-
-template<typename Type, bool isInt>
-struct QuotientBaseOp;
-
-template<typename Type>
-struct QuotientBaseOp<Type, false>
-{
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-  inline QuotientBaseOp(const QuotientBaseOp& other): other_(other.other_) {}
-  inline QuotientBaseOp(param1_type const& other): other_( static_cast<Type>(1) / other) {}
-  inline Type const operator() (param1_type const& a) const { return a * other_; }
-  Type const other_;
-};
-
-template<typename Type>
-struct QuotientBaseOp<Type, true>
-{
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-  inline QuotientBaseOp( QuotientBaseOp const& other): other_(other.other_) {}
-  inline QuotientBaseOp( param1_type other): other_(other) {}
-  inline Type const operator()( param1_type a) const { return a / other_; }
-  Type const other_;
-};
-
-/** @ingroup Functors
-  * @brief Template functor to divide a number by a fixed other one
-  *
-  * This functor is used to implement the quotient of a matrix by
-  * a number where the number type is not necessarily a floating point type.
-  */
-template<typename Type>
-struct QuotientOp: public QuotientBaseOp<Type, hidden::isInt<Type>::yes >
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline QuotientOp(param1_type const& other): QuotientBaseOp<Type, hidden::isInt<Type>::yes >(other) {}
-};
-/** @ingroup Functors
-  * @brief Template functor to add a number to a fixed other one
-  */
-template<typename Type>
-struct AddOp
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline AddOp(AddOp const& other): other_(other.other_) { }
-  inline AddOp(Type other): other_(other) { }
-  inline result_type operator() (param1_type const& a) const { return a + other_; }
-  const result_type other_;
-};
-/** @ingroup Functors
-  * @brief Template functor to add a number to a fixed other one
-  */
-template<typename Type>
-struct AddOppositeOp
-{
-  enum { NbParam_ = 1 };
-  typedef Type result_type;
-  typedef typename hidden::RemoveConst<Type>::Type param1_type ;
-
-  inline AddOppositeOp(AddOppositeOp const& other): other_(other.other_) { }
-  inline AddOppositeOp(Type other): other_(other) { }
-  inline result_type operator() (param1_type const& a) const { return other_ - a; }
-  const result_type other_;
-};
 /** @ingroup Functors
   * @brief Template functor which compute the square root of a number
   */
@@ -654,7 +850,7 @@ template<typename Type> struct SqrtOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator()(param1_type const& a) const { return std::sqrt(a); }
+  inline result_type operator()(param1_type const& a) const { return std::sqrt(a);}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the cosine of a number
@@ -665,7 +861,7 @@ template<typename Type> struct CosOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator() (param1_type const& a) const { return std::cos(a); }
+  inline result_type operator() (param1_type const& a) const { return std::cos(a);}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the sine of a number
@@ -676,7 +872,7 @@ template<typename Type> struct SinOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator() (param1_type const& a) const { return std::sin(a); }
+  inline result_type operator() (param1_type const& a) const { return std::sin(a);}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the tan of a number
@@ -687,7 +883,7 @@ template<typename Type> struct TanOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator() (param1_type const& a) const { return std::tan(a); }
+  inline result_type operator() (param1_type const& a) const { return std::tan(a);}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the arc cosine of a number
@@ -698,7 +894,7 @@ template<typename Type> struct AcosOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator() (param1_type const& a) const { return std::acos(a); }
+  inline result_type operator() (param1_type const& a) const { return std::acos(a);}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the arc sine of a number
@@ -709,7 +905,7 @@ template<typename Type> struct AsinOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator() (param1_type const& a) const { return std::asin(a); }
+  inline result_type operator() (param1_type const& a) const { return std::asin(a);}
 };
 /** @ingroup Functors
   * @brief Template functor to raise a number to a power
@@ -721,10 +917,12 @@ struct PowOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline PowOp(PowOp const& other): exponent_(other.exponent_) {}
+  inline PowOp(PowOp const& value): exponent_(value.exponent_) {}
   inline PowOp(Type exponent): exponent_(exponent) {}
-  inline result_type operator()(param1_type const& a) const  { return std::pow(a, exponent_); }
-  const Type exponent_;
+
+  inline result_type operator()(param1_type const& a) const  { return std::pow(a, exponent_);}
+  private:
+    const Type exponent_;
 };
 /** @ingroup Functors
   * @brief Template functor which compute the inverse of a number
@@ -736,7 +934,7 @@ struct InverseOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator() (param1_type const& a) const { return Type(1)/a; }
+  inline result_type operator() (param1_type const& a) const { return Type(1)/a;}
 };
 /** @ingroup Functors
   * @brief Template functor which compute the square of a number
@@ -748,7 +946,7 @@ struct SquareOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator() (param1_type const& a) const { return a*a; }
+  inline result_type operator() (param1_type const& a) const { return a*a;}
 };
 
 /** @ingroup Functors
@@ -761,7 +959,7 @@ struct CubeOp
   typedef Type result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type;
 
-  inline result_type operator() (param1_type const& a) const { return a*a*a; }
+  inline result_type operator() (param1_type const& a) const { return a*a*a;}
 };
 /** @ingroup Functors
   * @brief Template functor which cast a type to another type
@@ -773,8 +971,7 @@ struct CastOp
   typedef OtherType result_type;
   typedef typename hidden::RemoveConst<Type>::Type param1_type ;
 
-  inline result_type operator()(param1_type const& a) const
-  { return static_cast<OtherType>(a); }
+  inline result_type operator()(param1_type const& a) const { return static_cast<OtherType>(a);}
 };
 
 } // namespace STK
